using System;
using System.Drawing;
using System.Windows.Forms;
using VideoMotionDetectionLibrary;

namespace VideoMotionDetectionLibraryDemo
{
    public partial class MainFrm : Form
    {
        //Set Original Frame as Current Frame when new Frame is available.
        private bool refreshOriginalFrameNow = true;
        //Define VideoMotionDetection class.
        private VideoMotionDetection videoMotionDetection = null;
        //Define sensitive area.
        private Point[][] polygons = { new Point[] { new Point(5, 5), new Point(90, 5), new Point(90, 90), new Point(5, 90) }, new Point[] { new Point(100, 100), new Point(150, 110), new Point(70, 115) } };

        public MainFrm()
        {
            InitializeComponent();
            //Important message.
            MessageBox.Show("This demo will work properly only if you run it from executable file! Don't run this demo from Visual Studio!", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            InitializeVideoMotionDetection();
        }

        //This method runs after the program starts or new file loaded.
        private void InitializeVideoMotionDetection()
        {
            //Update controls.
            toleranceTextBox.Text = "Tolerance value is: " + toleranceTrackBar.Value.ToString();
            pointsToBeObjectTextBox.Text = "Points to be considered as an object value: " + pointsToBeObjectTrackBar.Value.ToString();
            minimalDistanceTextBox.Text = "Minimal distance between objects: " + minimalDistanceTrackBar.Value.ToString();
            modeComboBox.SelectedItem = "Compare Current Frame With Previous Frame";
            setCurrentFrameAsOriginalFrameButton.Enabled = false;
            //Clear all picture boxes.
            previewPictureBox.Image = null;
            averageObjectPointPictureBox.Image = null;
            objectIncludingRectanglePictureBox.Image = null;
            differenceFramePictureBox.Image = null;
            AdjustVideoWindowPlace();
            //AviPlayerControl settings.
            aviPlayerControl.SetStepInFrames(5);
            aviPlayerControl.OpenAVIFile(Application.StartupPath + @"\..\..\..\Demo.avi");
        }

        private void aviPlayerControl_FrameArrived(object sender, AVIPlayer.FrameCallBackEventArgs e)
        {
            if (videoMotionDetection == null) return;
            //Define moving Bitmaps.
            Bitmap previewBitmap = null;
            switch (videoMotionDetection.Mode)
            {
                case VideoMotionDetection.Modes.CompareCurrentFrameWithPreviousFrame:
                    previewBitmap = videoMotionDetection.GetPreviousFrameBitmap();
                    break;
                case VideoMotionDetection.Modes.CompareCurrentFrameWithOriginalFrame:
                    previewBitmap = videoMotionDetection.GetOriginalFrameBitmap();
                    break;
            }
            Bitmap averageObjectPointBitmap = (Bitmap)e.bitmap.Clone();
            Bitmap objectIncludingRectangleBitmap = (Bitmap)e.bitmap.Clone();
            Bitmap differenceFrameBitmap = videoMotionDetection.GetDifferenceFrameBitmap();
            /*
            When the new frame comes from the stream, it must be inserted into a Video Motion Detection queue,
            that contains two last frames. When it is inserted, the motion detection algorithm compares the two
            frames mentioned above in order to find moving object.
            */
            videoMotionDetection.InsertNewFrame(e.bitmap);
            //Set Original Frame as Current Frame when new Current Frame is available.
            if (refreshOriginalFrameNow) { videoMotionDetection.SetOriginalFrame(e.bitmap); refreshOriginalFrameNow = false; }
            //Draw all object's average points and rectangles. Draw sensitive area limiting polygons.
            if ((previewBitmap != null) && (averageObjectPointBitmap != null) && (objectIncludingRectangleBitmap != null) && (differenceFrameBitmap != null))
            {
                Graphics previewGraphics = Graphics.FromImage(previewBitmap);
                Graphics averageObjectPointGraphics = Graphics.FromImage(averageObjectPointBitmap);
                Graphics objectIncludingRectangleGraphics = Graphics.FromImage(objectIncludingRectangleBitmap);
                Graphics differenceFrameGraphics = Graphics.FromImage(differenceFrameBitmap);
                Motion motion = videoMotionDetection.Motion;
                for (int i = 0; i < motion.Count; i++)
                {
                    MovingObject movingObject = motion.GetMovingObject(i);
                    Point point = movingObject.AverageObjectPoint;
                    Rectangle rectangle = movingObject.ObjectIncludingRectangle;
                    averageObjectPointGraphics.DrawRectangle(new Pen(Color.White, 2), point.X, point.Y, 2, 2);
                    averageObjectPointGraphics.DrawString(movingObject.ObjectDensity.ToString("F2"), new Font("Arial", 10), new SolidBrush(Color.White), point);
                    objectIncludingRectangleGraphics.DrawRectangle(new Pen(Color.White, 2), rectangle);
                }
                if (videoMotionDetection.Polygons != null)
                {
                    foreach (Point[] polygon in videoMotionDetection.Polygons)
                    {
                        previewGraphics.DrawPolygon(new Pen(Color.Yellow), polygon);
                        averageObjectPointGraphics.DrawPolygon(new Pen(Color.Yellow), polygon);
                        objectIncludingRectangleGraphics.DrawPolygon(new Pen(Color.Yellow), polygon);
                        differenceFrameGraphics.DrawPolygon(new Pen(Color.Yellow), polygon);
                    }
                }
            }
            //Attach Bitmaps to Picture Boxes.
            previewPictureBox.Image = previewBitmap;
            averageObjectPointPictureBox.Image = averageObjectPointBitmap;
            objectIncludingRectanglePictureBox.Image = objectIncludingRectangleBitmap;
            differenceFramePictureBox.Image = differenceFrameBitmap;
        }

        private void aviPlayerControl_FrameSizeChanged(object sender, AVIPlayer.FrameSizeChangedEventArgs e)
        {
            //Make an instance of VideoMotionDetection class.
            videoMotionDetection = new VideoMotionDetection("Unregistered", "User");
            //Video Motion Detection tolerance.
            videoMotionDetection.Tolerance = toleranceTrackBar.Value;
            //Minimal number of connected points to be considered as an object.
            videoMotionDetection.MinimalAmountOfPointsToBeConsideredAsObject = pointsToBeObjectTrackBar.Value;
            //Set minimal distance between objects.
            videoMotionDetection.MinimalDistanceBetweenTwoObjects = minimalDistanceTrackBar.Value;
            //Set sensitive area.
            if (zoneLimitCheckBox.Checked) videoMotionDetection.Polygons = this.polygons; else videoMotionDetection.Polygons = null;
            //Set Video Motion Detection Mode, so that the last frame will be compared with previous one.
            videoMotionDetection.Mode = VideoMotionDetection.Modes.CompareCurrentFrameWithPreviousFrame;
            //Update sizes.
            previewPictureBox.Width = e.width;
            previewPictureBox.Height = e.height;
            averageObjectPointPictureBox.Width = e.width;
            averageObjectPointPictureBox.Height = e.height;
            objectIncludingRectanglePictureBox.Width = e.width;
            objectIncludingRectanglePictureBox.Height = e.height;
            differenceFramePictureBox.Width = e.width;
            differenceFramePictureBox.Height = e.height;
            AdjustVideoWindowPlace();
        }

        //Adjust video window position.
        private void AdjustVideoWindowPlace()
        {
            previewPictureBox.Left = Math.Max(0, (panel1.Width - previewPictureBox.Width) / 2);
            previewPictureBox.Top = Math.Max(0, (panel1.Height - previewPictureBox.Height) / 2);
            averageObjectPointPictureBox.Left = Math.Max(0, (panel2.Width - previewPictureBox.Width) / 2);
            averageObjectPointPictureBox.Top = Math.Max(0, (panel2.Height - previewPictureBox.Height) / 2);
            objectIncludingRectanglePictureBox.Left = Math.Max(0, (panel3.Width - previewPictureBox.Width) / 2);
            objectIncludingRectanglePictureBox.Top = Math.Max(0, (panel3.Height - previewPictureBox.Height) / 2);
            differenceFramePictureBox.Left = Math.Max(0, (panel4.Width - previewPictureBox.Width) / 2);
            differenceFramePictureBox.Top = Math.Max(0, (panel4.Height - previewPictureBox.Height) / 2);
        }

        /////////////////////////////////////////////////MAIN FORM////////////////////////////////////////////////////
        private void MainFrm_FormClosed(object sender, FormClosedEventArgs e) { GC.Collect(); }
        private void MainFrm_Resize(object sender, EventArgs e) { AdjustVideoWindowPlace(); }

        /////////////////////////////////////////////////TRACK BARS///////////////////////////////////////////////////
        private void toleranceTrackBar_Scroll(object sender, EventArgs e)
        {
            if (videoMotionDetection == null) return;
            videoMotionDetection.Tolerance = toleranceTrackBar.Value;
            toleranceTextBox.Text = "Tolerance value is: " + toleranceTrackBar.Value.ToString();
        }
        private void pointsToBeObjectTrackBar_Scroll(object sender, EventArgs e)
        {
            if (videoMotionDetection == null) return;
            videoMotionDetection.MinimalAmountOfPointsToBeConsideredAsObject = pointsToBeObjectTrackBar.Value;
            pointsToBeObjectTextBox.Text = "Points to be considered as an object value: " + pointsToBeObjectTrackBar.Value.ToString();
        }
        private void minimalDistanceTrackBar_Scroll(object sender, EventArgs e)
        {
            if (videoMotionDetection == null) return;
            videoMotionDetection.MinimalDistanceBetweenTwoObjects = minimalDistanceTrackBar.Value;
            minimalDistanceTextBox.Text = "Minimal distance between objects: " + minimalDistanceTrackBar.Value.ToString();
        }

        /////////////////////////////////////////////////CHECK BOX////////////////////////////////////////////////////
        private void zoneLimitCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (videoMotionDetection == null) return;
            if (zoneLimitCheckBox.Checked) videoMotionDetection.Polygons = this.polygons; else videoMotionDetection.Polygons = null;
        }

        ////////////////////////////////////////////////FILE SUBMENU//////////////////////////////////////////////////
        private void exitToolStripMenuItem_Click(object sender, EventArgs e) { Application.Exit(); }

        /////////////////////////////////////////////////HELP SUBMENU/////////////////////////////////////////////////
        private void helpTopicsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            System.Diagnostics.Process.Start(Application.StartupPath + @"\..\..\..\SoftCollectionVideoMotionDetectionLibraryForNET.chm");
        }

        private void visitWebSiteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            System.Diagnostics.Process.Start("http://www.soft-collection.com");
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AboutBox about = new AboutBox();
            about.ShowDialog();
        }

        ////////////////////////////////////////////////MODE CONTROLS//////////////////////////////////////////////////
        private void modeComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (videoMotionDetection == null) return;
            switch ((string)modeComboBox.SelectedItem)
            {
                case "Compare Current Frame With Previous Frame":
                    videoMotionDetection.Mode = VideoMotionDetection.Modes.CompareCurrentFrameWithPreviousFrame;
                    setCurrentFrameAsOriginalFrameButton.Enabled = false;
                    break;
                case "Compare Current Frame With Original Frame":
                    videoMotionDetection.Mode = VideoMotionDetection.Modes.CompareCurrentFrameWithOriginalFrame;
                    setCurrentFrameAsOriginalFrameButton.Enabled = true;
                    break;
            }
        }
        private void setCurrentFrameAsOriginalFrameButton_Click(object sender, EventArgs e) { refreshOriginalFrameNow = true; }
    }
}